/* uptime
 *
 * A trivial program to display the system uptime. 
 * 
 * Copyright (C) 2006 Ville Koskinen
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#include <stdio.h>
#include <string.h>
#include <be/kernel/OS.h>
#include <time.h>

/*! Return the current time as a string.
 *
 * \param buf The buffer to write the string to.
 * \param bufsize The size of the buffer.
 * \return The number of characters (excl. '\0') placed in \a buf, or 0 if
 * an error occurred.
 *
 * \sa strftime
 *
 * \note \a buf must be allocated by the caller.
 */
int
get_now_string(char *buf, int bufsize)
{
	time_t now = time(NULL);
	struct tm *tmp = localtime(&now);
	
	if (tmp == NULL)
		return 0;
	
	return strftime(buf, bufsize, "%H:%M:%S", 
	                localtime(&now));
}


/*! Return the uptime as a string.
 *
 * \param buf The buffer to write the string to.
 * \param bufsize The size of the buffer.
 * \return The number of characters (excl. '\0') placed in \a buf, or 0 if
 * the buffer was too small and the string was truncated.
 *
 * \note \a buf must be allocated by the caller.
 */
int
get_uptime_string(char *buf, int bufsize)
{
	/* Uptime in seconds. */
	uint32 uptime_s = (uint32)(system_time() / 1000000);

	int written = snprintf(buf, bufsize, "%ld days, %2d:%02d",
	                       (uint32)(uptime_s/(24*60*60)),
			       (uint)(uptime_s/(60*60)%24),
			       (uint)(uptime_s/60%60));
	
	if (written > bufsize)
		return 0;
	
	return written;
}


/*! Print a very short help message.
 */
void
print_help()
{
	static const char program_name[] = "uptime";
	
	printf(
"Usage: %s\n"
"\n"
"%s will print how long the system has been up and running.\n",
	program_name, program_name);
}

/*! Handle command line arguments and print information accordingly.
 *
 * \return 0 if everything was fine, 1 if an error occurred.
 *
 * If parameters are given, print a (very) short help message and exit.
 */
int 
main(int argc, char **argv)
{
	static const int STRING_MAX_LENGTH = 20;
	char now_string[STRING_MAX_LENGTH];
	char uptime_string[STRING_MAX_LENGTH];

	if (argc > 1) {
		print_help();
		return 0;
	}
	
	if (get_now_string(now_string, STRING_MAX_LENGTH) == 0) {
		fprintf(stderr, "Error: failed to fit current time to the"
		                "buffer. Exiting.\n");
		return 1;
	}

	if (get_uptime_string(uptime_string, STRING_MAX_LENGTH) == 0) {
		fprintf(stderr, "Error: failed to fit the uptime to the"
		                "buffer. Exiting.\n");
		return 1;
	}

	printf("%s up %s\n", now_string, uptime_string);

	return 0;
}

